const webpack = require('webpack');
const { join, basename } = require('path');
const fetch = require('node-fetch');
const { cloneDeep } = require('lodash');

exports.pluginOptionsSchema = ({ Joi }) => Joi.object({
  accessToken: Joi.string().required(),
  pageTemplatePath: Joi.string().default(null),
  ignoredPageSlugs: Joi.array().default([]),
  directoryPath: Joi.string().required(),
});

exports.createSchemaCustomization = ({ actions }) => {
  const { createTypes } = actions;
  const typeDefs = `
    type Nebo implements Node {
      components: [NeboComponent]
    }

    type NeboComponent {
      id: ID
      slug: String
      name: String
      type: String
      schema: JSON
      params: JSON
    }
  `;
  createTypes(typeDefs);
};

exports.sourceNodes = async ({ actions, createContentDigest }, { accessToken }) => {
  const { createNode } = actions;

  let componentJSON;
  try {
    const response = await fetch('https://app.nebohq.com/api/components', {
      headers: {
        'Content-Type': 'application/json',
        Accept: 'application/json',
        Authorization: `Bearer ${accessToken}`,
      },
    });
    componentJSON = await response.json();
  } catch (error) {
    // eslint-disable-next-line no-console
    console.error(error);
    throw new Error('Could not fetch Nebo Components');
  }

  const components = componentJSON.map((component) => ({
    ...component,
    internal: {
      type: 'NeboComponent',
    },
  }));

  createNode({
    id: 'nebo',
    components,
    pages: [],
    internal: {
      type: 'Nebo',
      contentDigest: createContentDigest({ components }),
    },
  });
};

exports.createPages = async ({ actions, graphql }, {
  pageTemplatePath,
  ignoredPageSlugs,
}) => {
  const { data } = await graphql(`
    query {
      nebo {
        components {
          id
          name
          slug
          type
          schema
          params
        }
      }
    }
  `);

  const pages = data.nebo.components.filter((component) => component.type === 'page');
  const schemas = data.nebo.components.reduce((acc, component) => {
    acc[component.slug] = component;
    acc[component.id] = component;
    return acc;
  }, {});

  if (!pageTemplatePath) return;

  const ignoreSlugs = new Set(ignoredPageSlugs);
  pages.forEach((page) => {
    const { slug } = page;
    if (ignoreSlugs.has(slug)) return;

    actions.createPage({
      path: slug.replace(/(^\/)|(\/$)/g, ''),
      component: require.resolve(pageTemplatePath),
      context: { page, schemas },
    });
  });
};

exports.onCreateWebpackConfig = ({ stage, getConfig, store }, { directoryPath }) => {
  if (!(['build-javascript', 'develop'].includes(stage))) return null;

  const gatsbyConfig = getConfig();
  const { program } = store.getState();

  const pluginsToIgnore = [
    'MiniCssExtractPlugin',
    'GatsbyWebpackStatsExtractor',
    'StaticQueryMapper',
  ];
  const filename = [basename(directoryPath).split('.').slice(0, -1), '.js'].join('');
  const config = {
    ...gatsbyConfig,
    entry: directoryPath,
    output: {
      path: join(program.directory, 'public'),
      filename,
    },
    module: {
      rules: [...cloneDeep(gatsbyConfig.module.rules)],
    },
    plugins: [...gatsbyConfig.plugins].filter((plugin) => (
      !pluginsToIgnore.includes(plugin.constructor && plugin.constructor.name)
    )),
    mode: stage === 'develop' ? 'development' : 'production',
    optimization: {},
  };

  return new Promise((resolve, reject) => {
    if (stage === 'develop') {
      webpack(config).watch({}, () => {});

      return resolve();
    }

    return webpack(config).run((err, stats) => {
      if (err) return reject(err);
      const errors = stats.compilation.errors || [];
      if (errors.length > 0) return reject(stats.compilation.errors);
      return resolve();
    });
  });
};
